import greenfoot.*;
import java.awt.Color;

/**
 * @author Ben Mullins
 * @oldAuthors Ben Mullins, Dylan Wilson, Chris Deneweth, Suphat Yoopensuk
 * 
 * @version 1.2.1
 * 
 * NOTE: Any version after 1.0b was created by Ben Mullins because this project was originally a CSC 101 project where there was a team.
 *       Afterwards, I continued to develop the game in my free time.
 *       
 * Also note that the changelog has been moved to the READ ME txt file because it was starting to take up a lot of space.
 */

public class Blob extends Actor {

    private GreenfootImage image1, image2, image3, image4, image5, image6, image7, image8, image9, image10, image11, image12;
    private int health, currentImageGreen, currentImageOrange, currentImageRed;
    private long imageSwapCounter;
    private Color white;
    private HealthStatus healthStat;

    public Blob(HealthStatus hStat) {
        
        healthStat = hStat;

        imageSwapCounter = System.currentTimeMillis();
        health = 100;

        currentImageGreen = 0;
        currentImageOrange = 2;
        currentImageRed = 4;
        white = Color.white;

        image1 = new GreenfootImage("Blob_down.png");
        image2 = new GreenfootImage("Blob_up.png");
        image3 = new GreenfootImage("BlobSlightDamage_down.png");
        image4 = new GreenfootImage("BlobSlightDamage_up.png");
        image5 = new GreenfootImage("BlobHighDamage_down.png");
        image6 = new GreenfootImage("BlobHighDamage_up.png");

        image7 = new GreenfootImage("Blob_downDMG.png");
        image8 = new GreenfootImage("Blob_upDMG.png");
        image9 = new GreenfootImage("BlobSlightDamage_downDMG.png");
        image10 = new GreenfootImage("BlobSlightDamage_upDMG.png");
        image11 = new GreenfootImage("BlobHighDamage_downDMG.png");
        image12 = new GreenfootImage("BlobHighDamage_upDMG.png");

        setImage(image1);
    }

    public void act() {

        moveAround();
        updateHealth();
    }

    public void moveAround() {

        animateBlob();
        detectColor();
        hasFlag();
    }

    /**
     *@animateBlob() is what keeps the images switching between its "down" and "up" positions.
     *The reason why there are three current image variables is because each color relates to how much health the character has left.
     */
    public void animateBlob() {

        if(health > 50) {
            if(System.currentTimeMillis() - imageSwapCounter > 200) {
                imageSwapCounter = System.currentTimeMillis();
                currentImageGreen++;
                if(currentImageGreen > 1) {
                    currentImageGreen = 0;
                }
                if(currentImageGreen == 0) {
                    setImage(image1);
                } else {
                    setImage(image2);
                }
            }
        } else if(health > 25 && health < 51) {
            if(System.currentTimeMillis() - imageSwapCounter > 200) {
                imageSwapCounter = System.currentTimeMillis();
                currentImageOrange++;
                if(currentImageOrange > 3) {
                    currentImageOrange = 2;
                }
                if(currentImageOrange == 2) {
                    setImage(image3);
                } else {
                    setImage(image4);
                }
            }
        } else if(health < 26) {
            if(System.currentTimeMillis() - imageSwapCounter > 200) {
                imageSwapCounter = System.currentTimeMillis();
                currentImageRed++;
                if(currentImageRed > 5) {
                    currentImageRed = 4;
                }
                if(currentImageRed == 4) {
                    setImage(image5);
                } else {
                    setImage(image6);
                }
            }
        }
    }

    /**
     * @updateHealth() adds health back to the player when a health pack is consumed
     */
    public void updateHealth() {

        Actor healthPack = getOneObjectAtOffset(0, 0, Health.class);
        GameOver gameOver = new GameOver();
        World world = getWorld();

        if(healthPack != null) {
            world.removeObject(healthPack);
            health += 10;
            healthStat.update("Health: " + health);
            Greenfoot.playSound("eating.wav");
        }

        if(health > 100) {
            health = 100;
            healthStat.update("Health: " + health);
        }

        if(health < 0) {
            health = 0;
            healthStat.update("Health: " + health);
        }
        if(health == 0) {
            world.addObject(gameOver, 350, 350);
            world.removeObject(this);
            Greenfoot.stop();
        }

        getHealth();
    }

    /**
     * @detectColor() allows the character to distinguish between the different colors on the background.
     * Since we did not create individual barriers for each wall, we just make it to where if the character
     *  comes in contact with a "wall" (aka the color black) it will stop, then lose some health.
     * 
     * Also, @detectColor() assigns how the player moves around.
     */
    public void detectColor() {

        World world = getWorld();
        GreenfootImage background = world.getBackground();

        if(Greenfoot.isKeyDown("shift")) {
            moveFast();
        } else {
            if(Greenfoot.isKeyDown("left") || Greenfoot.isKeyDown("a")) {
                if(background.getColorAt(getX() - 7, getY()).equals(white)) {
                    move(-2);
                    if(!background.getColorAt(getX() - 7, getY()).equals(white)) {
                        detectBlobColor();
                        health -= 5;
                        healthStat.update("Health: " + health);
                        Greenfoot.playSound("8BitHurt.wav");
                    }
                }
            }
            if(Greenfoot.isKeyDown("right") || Greenfoot.isKeyDown("d")) {
                if(background.getColorAt(getX() + 7, getY()).equals(white)) {
                    move(2);
                    if(!background.getColorAt(getX() + 7, getY()).equals(white)) {
                        detectBlobColor();
                        health -= 5;
                        healthStat.update("Health: " + health);
                        Greenfoot.playSound("8BitHurt.wav");
                    }
                }
            }
            if(Greenfoot.isKeyDown("up") || Greenfoot.isKeyDown("w")) {
                if(background.getColorAt(getX(), getY() - 4).equals(white)) {
                    setLocation(getX(), getY() - 2);
                    if(!background.getColorAt(getX(), getY() - 4).equals(white)) {
                        detectBlobColor();
                        health -= 5;
                        healthStat.update("Health: " + health);
                        Greenfoot.playSound("8BitHurt.wav");
                    }
                }
            }
            if(Greenfoot.isKeyDown("down") || Greenfoot.isKeyDown("s")) {
                if(background.getColorAt(getX(), getY() + 7).equals(white)) {
                    setLocation(getX(), getY() + 2);
                    if(!background.getColorAt(getX(), getY() + 7).equals(white)) {
                        detectBlobColor();
                        health -= 5;
                        healthStat.update("Health: " + health);
                        Greenfoot.playSound("8BitHurt.wav");
                    }
                }
            }
        }
    }

    /**
     * @moveFast() is the method that allows you to move faster whilst holding down Shift.
     */
    public void moveFast() {

        World world = getWorld();
        GreenfootImage background = world.getBackground();

        quickAnimate();

        if(Greenfoot.isKeyDown("left") || Greenfoot.isKeyDown("a")) {
            if(background.getColorAt(getX() - 7, getY()).equals(white)) {
                move(-4);
                if(!background.getColorAt(getX() - 7, getY()).equals(white)) {
                    detectBlobColor();
                    health -= 5;
                    healthStat.update("Health: " + health);
                    Greenfoot.playSound("8BitHurt.wav");
                }
            }
        }
        if(Greenfoot.isKeyDown("right") || Greenfoot.isKeyDown("d")) {
            if(background.getColorAt(getX() + 7, getY()).equals(white)) {
                move(4);
                if(!background.getColorAt(getX() + 7, getY()).equals(white)) {
                    detectBlobColor();
                    health -= 5;
                    healthStat.update("Health: " + health);
                    Greenfoot.playSound("8BitHurt.wav");
                }
            }
        }
        if(Greenfoot.isKeyDown("up") || Greenfoot.isKeyDown("w")) {
            if(background.getColorAt(getX(), getY() - 4).equals(white)) {
                setLocation(getX(), getY() - 4);
                if(!background.getColorAt(getX(), getY() - 4).equals(white)) {
                    detectBlobColor();
                    health -= 5;
                    healthStat.update("Health: " + health);
                    Greenfoot.playSound("8BitHurt.wav");
                }
            }
        }
        if(Greenfoot.isKeyDown("down") || Greenfoot.isKeyDown("s")) {
            if(background.getColorAt(getX(), getY() + 7).equals(white)) {
                setLocation(getX(), getY() + 4);
                if(!background.getColorAt(getX(), getY() + 7).equals(white)) {
                    detectBlobColor();
                    health -= 5;
                    healthStat.update("Health: " + health);
                    Greenfoot.playSound("8BitHurt.wav");
                }
            }
        }
    }
    
    /**
     * @quickAnimate() is kind of self explanatory. It animates the character 2x faster to make the sprinting look more realistic.
     */
    public void quickAnimate() {
    
        if(health > 50) {
            if(System.currentTimeMillis() - imageSwapCounter > 100) {
                imageSwapCounter = System.currentTimeMillis();
                currentImageGreen++;
                if(currentImageGreen > 1) {
                    currentImageGreen = 0;
                }
                if(currentImageGreen == 0) {
                    setImage(image1);
                } else {
                    setImage(image2);
                }
            }
        } else if(health > 25 && health < 51) {
            if(System.currentTimeMillis() - imageSwapCounter > 200) {
                imageSwapCounter = System.currentTimeMillis();
                currentImageOrange++;
                if(currentImageOrange > 3) {
                    currentImageOrange = 2;
                }
                if(currentImageOrange == 2) {
                    setImage(image3);
                } else {
                    setImage(image4);
                }
            }
        } else if(health < 26) {
            if(System.currentTimeMillis() - imageSwapCounter > 200) {
                imageSwapCounter = System.currentTimeMillis();
                currentImageRed++;
                if(currentImageRed > 5) {
                    currentImageRed = 4;
                }
                if(currentImageRed == 4) {
                    setImage(image5);
                } else {
                    setImage(image6);
                }
            }
        }
    }

    public void detectBlobColor() {

        if(health > 50) {
            switch(currentImageGreen) {
                case 0: setImage(image7); break;
                case 1: setImage(image8); break;
            }
        }
        if(health > 25 && health < 51) {
            switch(currentImageOrange) {
                case 2: setImage(image9); break;
                case 3: setImage(image10); break;
            }
        }
        if(health < 26) {
            switch(currentImageRed) {
                case 4: setImage(image11); break;
                case 5: setImage(image12); break;
            }
        }
    }

    /**
     * @hasFlag determines if the player has passed over the flag. If it has, then the game ends and you win.
     */
    public void hasFlag() {

        Actor flag = getOneObjectAtOffset(0, 0, Flag.class);
        YouWin win = new YouWin();
        World world = getWorld();

        if(flag != null) {
            world.addObject(win, 350, 350);
            Greenfoot.stop();
        }
    }

    /**
     * @getHealth() returns health
     */
    public int getHealth() {

        return health;
    }
}