import greenfoot.*;
import java.awt.Color;

/**
 * @author Ben Mullins
 * @oldAuthors Ben Mullins, Dylan Wilson, Chris Deneweth, Suphat Yoopensuk
 * 
 * @version 1.2.1
 * 
 * NOTE: Any version after 1.0b was created by Ben Mullins because this project was originally a CSC 101 project where there was a team.
 *       Afterwards, I continued to develop the game in my free time.
 *       
 * Also note that the changelog has been moved to the READ ME txt file because it was starting to take up a lot of space.
 */
public class Enemy extends Actor {

    private GreenfootImage image1, image2;
    private Color white;

    private int currentImage, direction;
    private long imageSwapCounter;

    private static final int RIGHT = 0;
    private static final int LEFT = 1;
    private static final int UP = 2;
    private static final int DOWN = 3;

    public Enemy() {

        image1 = new GreenfootImage("Enemy_in.png");
        image2 = new GreenfootImage("Enemy_out.png");

        white = Color.white;

        currentImage = 0;
        imageSwapCounter = System.currentTimeMillis();

        setImage(image1);
        setDirection(RIGHT);
    }

    public void act() {

        animateEnemy();

        if(canMove()) {
            moveAround();
            detectBlob();
        } else {
            switchDirection();
        }
    }    

    public void moveAround() {

        if(!canMove()) {
            return;
        }

        switch(direction) {
            case RIGHT: move(1); break;
            case LEFT: move(-1); break;
            case UP: setLocation(getX(), getY() - 1); break;
            case DOWN: setLocation(getX(), getY() + 1); break;
        }
    }

    public void animateEnemy() {

        if(System.currentTimeMillis() - imageSwapCounter > 100) {
            imageSwapCounter = System.currentTimeMillis();
            currentImage++;
            if(currentImage > 1) {
                currentImage = 0;
            }
            if(currentImage == 0) {
                setImage(image1);
            } else {
                setImage(image2);
            }
        }
    }

    public void detectBlob() {

        Actor blob = getOneObjectAtOffset(0, 0, Blob.class);
        World world = getWorld();
        GameOver gameOver = new GameOver();

        if(blob != null) {
            world.removeObject(blob);
            Greenfoot.playSound("Dead.wav");
            world.addObject(gameOver, 350, 350);
            Greenfoot.stop();
        }
    }

    public boolean canMove() {

        World world = getWorld();
        GreenfootImage background = world.getBackground();

        int x = getX();
        int y = getY();

        switch(direction) {
            case RIGHT: x += 6; break;
            case LEFT: x -= 6; break;
            case UP: y -= 6; break;
            case DOWN: y += 6; break;
        }

        if(!background.getColorAt(x, y).equals(white)) {
            return false;
        }
        return true;
    }

    public void switchDirection() {

        switch(direction) {
            case RIGHT: setDirection(DOWN); break;
            case DOWN: setDirection(UP); break;
            case UP: setDirection(LEFT); break;
            case LEFT: setDirection(RIGHT); break;
        }
    }

    public void setDirection(int direction) {

        this.direction = direction;

        switch(direction) {
            case RIGHT: move(1); break;
            case LEFT: move(-1); break;
            case UP: setLocation(getX(), getY() - 1); break;
            case DOWN: setLocation(getX(), getY() + 1); break;
        }
    }
}
